"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlobalStore = void 0;
const tslib_1 = require("tslib");
const react_dom_1 = tslib_1.__importDefault(require("react-dom"));
const react_native_global_state_hooks_1 = require("react-native-global-state-hooks");
class GlobalStore extends react_native_global_state_hooks_1.GlobalStore {
    /**
     * Create a new instance of the GlobalStore
     * @param {TState} state - The initial state
     * @param {TStateSetter} setterConfig - The actions configuration object (optional) (default: null) if not null the store manipulation will be done through the actions
     * @param {GlobalStoreConfig<TState, TMetadata>} config - The configuration object (optional) (default: { metadata: null })
     * @param {StateConfigCallbackParam<TState, TMetadata>} config.metadata - The metadata to pass to the callbacks (optional) (default: null)
     * @param {StateConfigCallbackParam<TState, TMetadata>} config.onInit - The callback to execute when the store is initialized (optional) (default: null)
     * @param {StateConfigCallbackParam<TState, TMetadata>} config.onStateChanged - The callback to execute when the state is changed (optional) (default: null)
     * @param {StateConfigCallbackParam<TState, TMetadata>} config.onSubscribed - The callback to execute when a subscriber is added (optional) (default: null)
     * @param {StateConfigCallbackParam<TState, TMetadata>} config.computePreventStateChange - The callback to execute when the state is changed to compute if the state change should be prevented (optional) (default: null)
     * @param {StateConfigCallbackParam<TState, TMetadata>} config.localStorageKey - The key to use to store the state in the localStorage (optional) (default: null) if not null the state will be stored in the localStorage
     * */
    constructor(state, _a = {}, setterConfig) {
        var _b;
        var { onInit: onInitConfig } = _a, config = tslib_1.__rest(_a, ["onInit"]);
        if (setterConfig === void 0) { setterConfig = null; }
        const decrypt = (config === null || config === void 0 ? void 0 : config.decrypt) === undefined ? (_b = config === null || config === void 0 ? void 0 : config.encrypt) !== null && _b !== void 0 ? _b : true : config === null || config === void 0 ? void 0 : config.decrypt;
        super(state, Object.assign({ metadata: null, encrypt: true, decrypt }, (config !== null && config !== void 0 ? config : {})), setterConfig);
        this.setLocalStorageValue = () => {
            const { localStorageKey } = this.config;
            let stateToStore = (0, react_native_global_state_hooks_1.formatToStore)(this.getStateClone(), {
                stringify: true,
            });
            const { encrypt } = this.config;
            if (encrypt) {
                const isEncryptCallback = typeof encrypt === 'function';
                const encryptCallback = (isEncryptCallback ? encrypt : (value) => btoa(value));
                stateToStore = encryptCallback(stateToStore);
            }
            localStorage.setItem(localStorageKey, stateToStore);
        };
        this.getLocalStorageValue = () => {
            const { localStorageKey } = this.config;
            let storedState = localStorage.getItem(localStorageKey);
            const { decrypt } = this.config;
            if (decrypt && storedState) {
                const isDecryptCallback = typeof decrypt === 'function';
                const decryptCallback = (isDecryptCallback ? decrypt : (value) => atob(value));
                storedState = decryptCallback(storedState);
            }
            return storedState;
        };
        /**
         * This method will be called once the store is created after the constructor,
         * this method is different from the onInit of the confg property and it won't be overriden
         */
        this.onInit = ({ setState, }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { localStorageKey } = this.config;
            if (!localStorageKey)
                return;
            const storedState = this.getLocalStorageValue();
            if (storedState === null) {
                this.setLocalStorageValue();
                return;
            }
            const jsonParsed = JSON.parse(storedState);
            const state = (0, react_native_global_state_hooks_1.formatFromStore)(jsonParsed);
            setState(state);
        });
        this.onStateChanged = () => {
            this.setLocalStorageValue();
        };
        /**
         * set the state and update all the subscribers,
         * In react web ReacDom allows to batch the state updates, this method will use the unstable_batchedUpdates method if it exists
         * @param {StateSetter<TState>} setter - The setter function or the value to set
         * @param {React.Dispatch<React.SetStateAction<TState>>} invokerSetState - The setState function of the component that invoked the state change (optional) (default: null) this is used to updated first the component that invoked the state change
         * */
        this.setState = ({ invokerSetState, state, }) => {
            // update the state
            this.state = state;
            const unstable_batchedUpdates = react_dom_1.default.unstable_batchedUpdates ||
                ((callback) => callback());
            unstable_batchedUpdates(() => {
                // execute first the callback of the component that invoked the state change
                invokerSetState === null || invokerSetState === void 0 ? void 0 : invokerSetState(state);
                // update all the subscribers
                this.subscribers.forEach((setState) => {
                    if (setState === invokerSetState)
                        return;
                    setState(state);
                });
            });
        };
        const parameters = this.getConfigCallbackParam({});
        this.onInit(parameters);
        onInitConfig === null || onInitConfig === void 0 ? void 0 : onInitConfig(parameters);
    }
}
exports.GlobalStore = GlobalStore;
